@@ -25,6 +25,8 @@ import glob
 import StringIO
 import gzip
 import shutil
+import sys
+import traceback
 
 
 month_names = {"Jan": 1, "Feb": 2, "Mar": 3, "Apr": 4, "May": 5, "Jun": 6,
@@ -438,7 +440,7 @@ class DownloadQueue:
             urllib.urlretrieve(url, filename)
         except Exception, ex:
             if self.__callback:
-                self.__callback("error", ex)
+                self.__callback(exception = ex)
 
 
     def wait_completion(self):
@@ -512,9 +514,14 @@ class Item:
             self.__download_queue = DownloadQueue(self.__report)
 
 
-    def __report(self, action = None, item = None):
+    def __report(self, action = None, item = None, exception = None):
+        details = None
+        if exception:
+            action = "error"
+            item = exception
+            details = ''.join(traceback.format_tb(sys.exc_info()[2]))
         if self.__callback:
-            self.__callback((action, item, None))
+            self.__callback((action, item, None, details))
 
 
     def __calc_the_tag(self, tag):
@@ -580,7 +587,7 @@ class Item:
                     a["href"] = item.save()
                     return a
         except Exception, ex:
-            self.__report("error", ex)
+            self.__report(exception = ex)
 
 
     def __find_single_page(self, html, base, referer):
@@ -593,7 +600,7 @@ class Item:
                     (new_html, new_base, real_url) = self.__get_page_html(url, headers={"Referer": referer})
                     return (new_html, new_base, True)
         except Exception, ex:
-            self.__report("error", ex)
+            self.__report(exception = ex)
         return (html, base, False)
 
 
@@ -607,7 +614,7 @@ class Item:
                     (new_html, new_base, real_url) = self.__get_page_html(url, headers={"Referer": referer})
                     return (new_html, new_base, get_body_or_html(new_html))
         except Exception, ex:
-            self.__report("error", ex)
+            self.__report(exception = ex)
         return (html, base, None)
 
 
@@ -663,7 +670,7 @@ class Item:
                 tag[attr] = self.__download_queue.queue(url, 
                     self.__get_cache_directory(), self.__cache_path)
             except Exception, ex:
-                self.__report("error", ex)
+                self.__report(exception = ex)
 
 
     def __cache_urls(self, tags, attr, base):
@@ -703,6 +710,8 @@ class Item:
         [root.append(tag) for tag in self.__contents]
         if not filename:
             filename = uuid.uuid1().hex + ".html"
+        if not os.path.exists(self.__path):
+            os.makedirs(self.__path)
         f = open(os.path.join(self.__path, filename), "w")
         f.write(html.prettify())
         f.close()
@@ -796,9 +805,14 @@ class Feed:
         self.new_item_count = 0
 
 
-    def __report(self, action = None, item = None, percent = None):
+    def __report(self, action = None, item = None, percent = None, exception = None):
+        details = None
+        if exception:
+            action = "error"
+            item = exception
+            details = ''.join(traceback.format_tb(sys.exc_info()[2]))
         if self.__callback:
-            self.__callback((action, item, percent))
+            self.__callback((action, item, percent, details))
 
 
     def __unquote_special(self, str):
@@ -1036,11 +1050,11 @@ class Feed:
                     self.__add_item(div, to_process[i], 20 + (i * 60 / count))
                     self.new_item_count += 1
                 except Exception, ex:
-                    self.__report("error", ex)
+                    self.__report(exception = ex)
             self.__report("downloading images", "", 90)
             self.__download_queue.wait_completion()
         except BaseException, ex:
-            self.__report("error", ex)
+            self.__report(exception = ex)
             raise
         finally:
             if self.new_item_count:
